 /*
  * lsmlib, STRASSER Michel may 2001
  *
  * This product is placed under GPL
  * Please see COPYING for further 
  * informations
  * ********************************
  *
  * this is the lsm.c file
  * *******************************/


#include <stdio.h>

#include <sys/types.h> /* for opendir , stat, getpwent, endpwent */
#include <dirent.h>  /* for opendir */
#include <stdlib.h>
#include <string.h> /* for strlen, strcpy, ... */

#include <sys/stat.h> /* for stat */
#include <unistd.h> /* for stat */

#include <pwd.h> /* to handle with users */
#include <grp.h> /* to handle with groups */  

#include <time.h> /* date of modif. ... */

#include <libxml/tree.h>
#include <errno.h>

#define LSM_COPYRIGHT "Strasser Michel, may 2001, this product is placed under\
 GPL, please see COPYING for further information"

#define LSM_GENERATED "Generated by lsmlib"
#define LSM_MAXPATHSIZE 1024
#define LSM_MAXUSERNAMESIZE 256
#define LSM_MAXGROUPNAMESIZE 256
#define LSM_MAXSIZEOFSIZE 256
#define LSM_MAXTIMESIZE 30
#define LSM_MAXAUTHSIZE 10

int addPath(const char *tree, const char *flower, char fullName[LSM_MAXPATHSIZE]){
  if ((strlen(tree) + strlen(flower) + 2) > LSM_MAXPATHSIZE) {
    fprintf(stderr, "Error addPath : path too long");
    return(1);
  }

  strcpy(fullName, tree);
  strcat(fullName, "/");
  strcat(fullName, flower); /* fr : j'ai oubli comment on dit feuille en anglais */

  return(0);
}

int mode_t2string (mode_t umt, char name[LSM_MAXAUTHSIZE]) {
  name[0]=(S_IRUSR & umt)?'r':'-';
  name[1]=(S_IWUSR & umt)?'w':'-';
  name[2]=(S_IXUSR & umt)?'x':'-';
  name[3]=(S_IRGRP & umt)?'r':'-';
  name[4]=(S_IWGRP & umt)?'w':'-';
  name[5]=(S_IXGRP & umt)?'x':'-';
  name[6]=(S_IROTH & umt)?'r':'-';
  name[7]=(S_IWOTH & umt)?'w':'-';
  name[8]=(S_IXUSR & umt)?'x':'-';
  name[9]='\0';
  return(0);
}

int time_t2string (time_t td, char name[LSM_MAXTIMESIZE]) {
  int i=0;
  int continuing=1;
  /* we need to delete the '\n' character */
  strcpy(name, ctime(&td));
  for (i=0; (i<LSM_MAXTIMESIZE)&&continuing; i++) {
    if (name[i]== '\n') {
      name[i]='\0';
      continuing=0;
    }
  }
  return(0);
}

int int2string (int size, char name[LSM_MAXSIZEOFSIZE]) {
  if(sprintf (name, "%d", size)==-1){
    fprintf(stderr, "Error sprintf, value may be trunked");
    return(1);
  }
  return(0);
}

int off_t2string (off_t size, char name[LSM_MAXSIZEOFSIZE]) {
  if (int2string ((int)size, name)!=0) {
    fprintf(stderr, "Error int2string");
    return(1);
  }
  return(0);
}

int uid2string (uid_t uid, char name[LSM_MAXUSERNAMESIZE]) {
  struct passwd *myPwEnt;
  myPwEnt = getpwuid(uid);
  if (strlen(myPwEnt->pw_name)>LSM_MAXUSERNAMESIZE){
    fprintf(stderr, "Error strcpy, user name too long");
    return(1);
  }
  strcpy(name, myPwEnt->pw_name);
  return(0);
}

int gid2string (gid_t gid, char name[LSM_MAXGROUPNAMESIZE]) {
  struct group *myGrEnt;
  myGrEnt = getgrgid(gid);
  if (strlen(myGrEnt->gr_name)>LSM_MAXGROUPNAMESIZE){
    fprintf(stderr, "Error strcpy, group name too long");
    return(1);
  }
  strcpy(name, myGrEnt->gr_name);
  return(0);
}


 /* ********** scanDir **********************
  * 
  * scanDir analyse a directory and complete the
  * associated dirNode. It is a recursive function.
  * *****/

int scanDir (const char *dir, xmlNodePtr dirNode) {  
  DIR *myDir;
  struct dirent *myDirEnt;
  struct stat myStat;
  xmlNodePtr myTreeElement;
  char fullName[LSM_MAXPATHSIZE];
  char userName[LSM_MAXUSERNAMESIZE];
  char groupName[LSM_MAXGROUPNAMESIZE];
  char sizeName[LSM_MAXSIZEOFSIZE];
  char atimeName[LSM_MAXTIMESIZE]; /* access */
  char mtimeName[LSM_MAXTIMESIZE]; /* modified */
  char ctimeName[LSM_MAXTIMESIZE]; /* creation */
  char authName[LSM_MAXAUTHSIZE]; /* authorisation / permission */
  
  if ((myDir = opendir (dir))==NULL){
    fprintf(stderr, "Error opendir, dir : %s\n", dir);
    return(1);
  }
  while ((myDirEnt=readdir(myDir))!=NULL){
    addPath(dir, myDirEnt->d_name, fullName);
    if(stat(fullName, &myStat) != 0 ) {
      fprintf(stderr, "Error stat, dir : %s\n", fullName);
      perror("stat");
      return(1);
    }
    if(S_ISDIR(myStat.st_mode)){
      myTreeElement = xmlNewChild(dirNode, NULL, "directory", NULL);      
      if (strcmp(myDirEnt->d_name, ".")  &&  strcmp(myDirEnt->d_name, "..")) { 
        /* Recursive call of function */
        if (scanDir(fullName, myTreeElement)!=0){
          /* Nothing to do in particulary
	   * It is the way of life that sometimes
	   * you can not stat a dir element 
	   * if you do not have permission in the current directory
	   * *****/
	  fprintf(stderr, "Warning scandir, scandir %s failed\n", fullName); 
	}
      }
    } else {
      myTreeElement = xmlNewChild(dirNode, NULL, "file", NULL);
    } 
    xmlSetProp(myTreeElement, "name", myDirEnt->d_name);
    if (uid2string(myStat.st_uid, userName)!=0) {
      fprintf(stderr, "Error uid2string, file : %s\n", fullName);
      return(1);
    }
    xmlSetProp(myTreeElement, "user", userName);
    if (gid2string(myStat.st_gid, groupName)!=0) {
      fprintf(stderr, "Error gid2string, file : %s\n", fullName);
      return(1);
    }
    xmlSetProp(myTreeElement, "group", groupName);
    if (off_t2string(myStat.st_size, sizeName)!=0) {
      fprintf(stderr, "Error off_t2string, file : %s\n", fullName);
      return(1);	    
    }
    xmlSetProp(myTreeElement, "size", sizeName);
    if (time_t2string(myStat.st_atime, atimeName)!=0) {
      fprintf(stderr, "Error time_t2string, file : %s\n", fullName);
      return(1);	    
    }
    xmlSetProp(myTreeElement, "atime", atimeName);
    if (time_t2string(myStat.st_mtime, mtimeName)!=0) {
      fprintf(stderr, "Error time_t2string, file : %s\n", fullName);
      return(1);	    
    }
    xmlSetProp(myTreeElement, "mtime", mtimeName);
    if (time_t2string(myStat.st_ctime, ctimeName)!=0) {
      fprintf(stderr, "Error time_t2string, file : %s\n", fullName);
      return(1);	    
    }
    xmlSetProp(myTreeElement, "ctime", ctimeName);
    if (mode_t2string(myStat.st_mode, authName)!=0) {
      fprintf(stderr, "Error umode_t2string, file : %s\n", fullName);
      return(1);	    
    }
    xmlSetProp(myTreeElement, "auth", authName);   
       
  }
  closedir(myDir);
  return(0);
}

 /* ********** lsm_getTree ******************
  * 
  * this function is exported by the library
  * *****/

xmlDocPtr lsm_getTree (const char *dir) {
  xmlDocPtr doc;
  xmlNodePtr filesRoot;
  
  doc = xmlNewDoc("1.0");
  doc->children = xmlNewDocNode(doc, NULL, "lsmtree", NULL);
  xmlSetProp(doc->children, "copyright", LSM_COPYRIGHT);
  xmlSetProp(doc->children, "generated", LSM_GENERATED);
  
  filesRoot = xmlNewChild(doc->children, NULL, "files", dir);
  
  if (scanDir(dir, filesRoot)!=0){
    fprintf(stderr, "Error scanDir\n");	  
    return(NULL);
  }
  
  return (doc);
}

